<?php

use App\Models\Enums\StudyAccessFlags;
use App\Models\Site;
use App\Models\User;
use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    public function up(): void
    {
        Schema::create('studies', function (Blueprint $table) {
            $table->id();
            $table->string('orthanc_uid')->unique();
            $table->boolean('is_active')->default(true);
            $table->unsignedTinyInteger('study_priority')->default(0);
            $table->string('patient_id')->nullable();
            $table->string('patient_name');
            $table->string('patient_sex');
            $table->date('patient_birthdate')->nullable();
            $table->string('study_instance_uid')->unique();
            $table->string('institution_name');
            $table->string('accession_number')->nullable();
            $table->string('study_description')->nullable();
            $table->dateTime('study_date')->index();
            $table->dateTime('receive_date')->index();
            $table->dateTime('report_date')->nullable();
            $table->foreignIdFor(Site::class)->constrained()->onDelete('cascade');
            $table->unsignedTinyInteger('report_status')->default(0);
            $table->string('study_modality', 4);
            $table->foreignIdFor(User::class, 'assigned_physician_id')->nullable()->constrained()->onDelete('set null');
            $table->foreignIdFor(User::class, 'interpreting_physician_id')->nullable()->constrained()->onDelete('set null');
            $table->foreignIdFor(User::class, 'referring_physician_id')->nullable()->constrained()->onDelete('set null');
            $table->unsignedTinyInteger('access_flags')->default(StudyAccessFlags::None->value);
            $table->string('access_password')->nullable();
            $table->timestamps();

            $table->index(['referring_physician_id', 'receive_date']);
            $table->index(['site_id', 'receive_date']);
            $table->index(['site_id', 'accession_number']);
            $table->index(['site_id', 'report_status', 'priority', 'receive_date']);
            $table->index(['assigned_physician_id', 'report_status', 'priority', 'receive_date']);
        });
    }

    public function down(): void
    {
        Schema::dropIfExists('studies');
    }
};
